#ifndef DNA_BINDING_H
#define DNA_BINDING_H

/******************************************************************************

Coral Tree dna/binding.h
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include <gpl/gpl.h>
#include <dna/bus.h>

/***********************************************************************
class DNA_Point

Network-layer peer identification.
***********************************************************************/
class DNA_Point : public GPL_Object, public GPL_Depictable
{
	public:
		DNA_Point(void) {}
		virtual ~DNA_Point(void) {}

		virtual long Compare(DNA_Point *p) const
		{
			p = p;
			return 0;
		}

		// GPL_Depictable
		virtual GPL_Boolean Print(GPL_String & string,
									const GPL_Verbosity verbosity) const
		{
			string = string;
			GPL_Verbosity v = verbosity; v = v;
			return FALSE;
		}
		virtual GPL_Boolean Parse(const GPL_String & string)
		{
			GPL_String s = string; s = s;
			return FALSE;
		}

		// GPL_Object
		GPL_OBJECT_ABSTRACT_INTERFACE(DNA_Point);
};

#define	DNA_POINT_CLONE(x)	(((DNA_Point *) GPL_BAD_ADDRESS == (x)) ? ((DNA_Point *) GPL_BAD_ADDRESS) : ((DNA_Point *) (x)->Clone()))

/***********************************************************************
class DNA_BindingImp

Network-layer peer binding implementation.
***********************************************************************/
class DNA_BindingImp : public DNA_BusImp
{
	public:
		DNA_BindingImp(void) {}
		virtual ~DNA_BindingImp(void) {}

		virtual DNA_Point *GetDestination(void) const
		{
			return (DNA_Point *) GPL_BAD_ADDRESS;
		}

		virtual void SetDestination(DNA_Point *p) {p=p;}

		virtual GPL_Boolean Open(DNA_Error & error)
		{
			error = error;
			return FALSE;
		}
		virtual GPL_Boolean Bind(DNA_Point *point, DNA_Error & error)
		{
			point = point;
			error = error;
			return FALSE;
		}
		virtual GPL_Boolean Close(DNA_Error & error)
		{
			error = error;
			return FALSE;
		}

		// GPL_Object
		GPL_OBJECT_ABSTRACT_INTERFACE(DNA_BindingImp);
};

/***********************************************************************
enum DNA_BindingState
***********************************************************************/
enum DNA_BindingState
{
	DNA_BINDING_VALID	=	DNA_BUS_VALID,
	DNA_BINDING_OPEN	=	0x00000002,
	DNA_BINDING_BOUND	=	0x00000004
};

/***********************************************************************
class DNA_Binding

Network-layer peer binding.
***********************************************************************/
class DNA_Binding : public DNA_Bus
{
	public:
		DNA_Binding(void) {}
		DNA_Binding(const DNA_Binding & b) { Attach(b); }
		virtual ~DNA_Binding(void) {}

		DNA_Binding & operator=(const DNA_Binding & b) { Detach(); Attach(b); return *this; }
		GPL_Boolean operator==(const DNA_Binding & b) const { return 0 == Compare(b); }

		virtual GPL_Boolean IsOpen(void) const;
		virtual GPL_Boolean IsBound(void) const;
		virtual DNA_BindingImp *GetBindingImp(void) const
		{
			return (DNA_BindingImp *) DNA_Bus::GetImp();
		}
		virtual DNA_Point *GetSource(void) const;
		virtual DNA_Point *GetDestination(void) const;

		virtual void SetImp(DNA_BusImp *i);
		virtual void SetSource(DNA_Point *p);
		virtual void SetDestination(DNA_Point *p);

		virtual void Reset(void);
		virtual GPL_Boolean Open(void);
		virtual GPL_Boolean Bind(void);
		virtual GPL_Boolean Close(void);
		virtual GPL_Boolean Read(void *buffer, const unsigned long size,
							unsigned long & length);
		virtual GPL_Boolean Read(void *buffer, const unsigned long size,
							const long timeout, unsigned long & length);
		virtual GPL_Boolean Write(void *buffer, const unsigned long size,
							unsigned long & length);

		// GPL_Depictable
		virtual GPL_Boolean Print(GPL_String & string,
								const GPL_Verbosity verbosity) const;

		// GPL_Object
		GPL_OBJECT_INTERFACE(DNA_Binding);

	protected:
		virtual void Detach(void);
};

/******************************************************************************
class DNA_BindingRep

Network-layer peer binding representation.
******************************************************************************/
class DNA_BindingRep : public DNA_BusRep
{
	public:
		DNA_Point *source;
		DNA_Point *destination;

		DNA_BindingRep(void) : source((DNA_Point *) GPL_BAD_ADDRESS),
								destination((DNA_Point *) GPL_BAD_ADDRESS) {}
		virtual ~DNA_BindingRep(void);
};

#endif /* DNA_BINDING_H */
