
/******************************************************************************

Coral Tree dna/ipmplex.cc
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include <dna/dna.h>

DNA_IPMultiplexor::DNA_IPMultiplexor(void) : timeout(1)
{
}

DNA_IPMultiplexor::~DNA_IPMultiplexor(void)
{
}

unsigned long DNA_IPMultiplexor::Wait(const long t,
										GPL_List<DNA_IPMultiplexable> & ready)
{
	// Initialize the ready list.
	ready.Clear();

	// Initialize the file descriptor sets.
	FD_ZERO(&readfds);
	FD_ZERO(&writefds);
	FD_ZERO(&exceptfds);

	// Initialize the timeout.
	struct timeval to;
	struct timeval *top = (struct timeval *) 0;
	if (DNA_MULTIPLEXOR_BLOCK != t)
	{
		to.tv_sec = (int) (t / 1000);
		to.tv_usec = 1000 * (t - (1000 * to.tv_sec));
		top = &to;
	}

	// For every through the multiplexable bindings.
	int nfds = 0;
	register const unsigned long n = multiplexables.GetCount();
	register unsigned long i = 0;
	while (n > i)
	{
		// Get the IP socket handle.
		register const SOCKET_HANDLE handle = multiplexables[i].handle;
		if (INVALID_SOCKET_HANDLE != handle)
		{
			// Determine the number of file descriptors to check.
			if ((unsigned long) nfds <= handle)
			{
				nfds = 1 + handle;
			}

			// Add this handle to the file descriptor sets based on its flags.
			register const unsigned long flags = multiplexables[i].flags;
			if (flags & DNA_IPMULTIPLEXABLE_READ)
			{
				FD_SET(handle, &readfds);
			}
			if (flags & DNA_IPMULTIPLEXABLE_WRITE)
			{
				FD_SET(handle, &writefds);
			}
			if (flags & DNA_IPMULTIPLEXABLE_EXCEPTION)
			{
				FD_SET(handle, &exceptfds);
			}
		}

		++i;
	}

	// Do the select().
	int rc;
	rc = DNA_SELECT(nfds, &readfds, &writefds, &exceptfds, top);
	// If no handles had activity...
	if (0 == rc)
	{
		// Timed out.
		return 0;
	}

	// If an error occurred...
	if (-1 == rc)
	{
		wbaLog("DNA WARNING", "select failed in multiplexor");
		return 0;
	}

	// If one or more handles have activity on them...
	i = 0;
	unsigned long num = 0;
	DNA_IPMultiplexable multiplexable;
	while (n > i)
	{
		// Get the IP socket handle.
		register const SOCKET_HANDLE handle = multiplexables[i].handle;
		if (INVALID_SOCKET_HANDLE != handle)
		{
			// Determine what kind of activity is waiting for the channel.
			multiplexable.handle = handle;
			multiplexable.flags = 0;
			if (FD_ISSET(handle, &readfds))
			{
				multiplexable.flags |= DNA_IPMULTIPLEXABLE_READ;
			}
			if (FD_ISSET(handle, &writefds))
			{
				multiplexable.flags |= DNA_IPMULTIPLEXABLE_WRITE;
			}
			if (FD_ISSET(handle, &exceptfds))
			{
				multiplexable.flags |= DNA_IPMULTIPLEXABLE_EXCEPTION;
			}

			// If the handle has any activity whatsoever...
			if (multiplexable.flags)
			{
				// Add this binding to the ready list.
				ready.Append(multiplexable);
				++num;
			}
		}

		++i;
	}
	return num;
}

void DNA_IPMultiplexor::Add(SOCKET_HANDLE handle, const unsigned long flags)
{
	if (INVALID_SOCKET_HANDLE == handle)
	{
		return;
	}

	register const unsigned long n = multiplexables.GetCount();
	for (register unsigned long i = 0; n > i; ++i)
	{
		if (handle == multiplexables[i].handle)
		{
			multiplexables[i].flags = flags;
			return;
		}
	}

	DNA_IPMultiplexable multiplexable(handle, flags);
	multiplexables.Append(multiplexable);
}

void DNA_IPMultiplexor::Remove(SOCKET_HANDLE handle)
{
	for (register unsigned long i = 0; multiplexables.GetCount() > i; ++i)
	{
		if (handle == multiplexables[i].handle)
		{
			multiplexables.Remove(i);
		}
	}
}

