/******************************************************************************

Coral Tree dna/iserver.cc
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include <wba/wba.h>
#include <dna/dna.h>

/******************************************************************************
class DNA_IterativeServer
******************************************************************************/
DNA_IterativeServer::DNA_IterativeServer()
{
}

/******************************************************************************
GPL_Boolean DNA_IterativeServer::Initialize(void)
******************************************************************************/
GPL_Boolean DNA_IterativeServer::Initialize(void)
{
	if(!(DNA_Server::Initialize()))
	{
		return FALSE;
	}

	register DNA_TCPConnectionImp *imp =
		(DNA_TCPConnectionImp *) connection.GetConnectionImp();

	// Set up the multiplexor.
	connections.Insert(imp->GetSocketHandle(), connection);
	multiplexor.Add(imp->GetSocketHandle());

	return TRUE;
}

/******************************************************************************
void DNA_IterativeServer::Finalize(void)
******************************************************************************/
void DNA_IterativeServer::Finalize(void)
{
	DNA_Server::Finalize();
}

/******************************************************************************
void DNA_IterativeServer::Wait(void)
******************************************************************************/
void DNA_IterativeServer::Wait(void)
{
	// If there is activity on the multiplexor...
	if (0 < multiplexor.Wait(ready))
	{
		// Check the ready list for active channels.
		DNA_Message message;
		SOCKET_HANDLE handle;
		register const unsigned long n = ready.GetCount();
		for (unsigned long i = 0; n > i; ++i)
		{
			if (!(DNA_IPMULTIPLEXABLE_READ & ready[i].flags))
			{
				// Only reads should occur.
				wbaLog("DNA WARNING", "Invalid response from multiplexor");
				break;
			}

			// Check for a new connection request on the listen channel.
			handle = ready[i].handle;
			if (DNA_GET_HANDLE(connection) == handle)
			{
				DNA_Connection dummy;
				if (connections[handle].Accept(dummy))
				{
					connections[DNA_GET_HANDLE(dummy)] = dummy;
					multiplexor.Add(DNA_GET_HANDLE(dummy),
									DNA_IPMULTIPLEXABLE_READ);
					AcceptGoodConnection(DNA_GET_HANDLE(dummy));
				}
			}

			// Incoming message on existing channel.
			else
			{
				if (!connections[handle].IsValid())
				{
					continue;
				}
				// Receive the message from the connection.
				// NOTE: DNA_Message::Receive() does a clear
				// prior to message reception.
				if (!message.Receive(&(connections[handle]),GetTimeout()))
				{
					// If this is an irrecoverable error...
					const DNA_ErrorType type =
						(DNA_ErrorType) connections[handle].GetError().GetType();
					if (DNA_ERROR_NONE != type ||
						DNA_ERROR_INSUFFICIENT_MEMORY != type ||
						DNA_ERROR_NO_DATA_AVAILABLE != type ||
						DNA_ERROR_INTERRUPTED_CALL != type)
					{
						// Allow derived classes to handle the bad connection.
						DropBadConnection((GPL_Opaque) handle);
					}
					continue;
				}
				const GPL_String image = message.ToString();
				wbaLog("DNA LOG", "RCV >%s<", image.GetData());

				// Allow derived classes to process the message.
				ProcessMessage(message, (GPL_Opaque) handle);
			} // else
		} // for
	} // if Wait()
}

/***********************************************************************
void DNA_IterativeServer::Reject(const GPL_Opaque key)
***********************************************************************/
void DNA_IterativeServer::Reject(const GPL_Opaque key)
{
	// Validate the key.
	const SOCKET_HANDLE handle = (SOCKET_HANDLE) key;
	if (connections.Contains(handle))
	{
		multiplexor.Remove(handle);
	}

	DNA_Server::Reject(key);
}


