/******************************************************************************

Coral Tree dna/tcp.cc
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include <gpl/gpl.h>
#include <dna/dna.h>

/******************************************************************************
class DNA_TCPConnectionImp
******************************************************************************/
DNA_TCPConnectionImp::DNA_TCPConnectionImp(void)
	: destination((DNA_IPPoint *) 0), backLogSet(FALSE)
{
}

DNA_TCPConnectionImp::~DNA_TCPConnectionImp(void)
{
}

long DNA_TCPConnectionImp::Compare(const DNA_BusImp *imp) const
{
	if (GetClassId() != imp->GetClassId())
	{
		return GetClassId() - imp->GetClassId();
	}
	return socket.GetSocketHandle() - ((DNA_TCPConnectionImp *) imp)->socket.GetSocketHandle();
}

void DNA_TCPConnectionImp::SetDestination(DNA_Point *p)
{
	if ((DNA_Point *) 0 != p)
	{
		if (DNA_IPPoint::GetClassIdStatic() == p->GetClassId())
		{
			destination = (DNA_IPPoint *) p;
		}
	}
}

void DNA_TCPConnectionImp::SetBackLog(const unsigned long b)
{
	socket.SetBackLog(b);

	backLogSet = TRUE;
}

GPL_Boolean DNA_TCPConnectionImp::Accept(DNA_ConnectionImp *i, DNA_Error & error)
{
	if ((DNA_ConnectionImp *) 0 == i || GetClassId() != i->GetClassId())
	{
		error.Set(DNA_ERROR_BAD_IMPLEMENTATION, 0);
		return FALSE;
	}

	// If no back log has been set...
	if (!backLogSet)
	{
		SetBackLog(DNA_DEFAULT_TCP_BACK_LOG);
	}

	// Prepare a new destination.
	register DNA_IPPoint *p = new DNA_IPPoint();
	if ((DNA_IPPoint *) GPL_BAD_ADDRESS == p)
	{
		gplLog("DNA ERROR", "Out of memory");
		return FALSE;
	}

	if (socket.Accept(p, ((DNA_TCPConnectionImp *) i)->socket, error))
	{
		// Set the destination.
		i->SetDestination(p);
		return TRUE;
	}

	delete p;
	return FALSE;
}

GPL_Boolean DNA_TCPConnectionImp::Connect(DNA_Error & error)
{
	return socket.Connect(destination, error);
}

GPL_Boolean DNA_TCPConnectionImp::Disconnect(DNA_Error & error)
{
	return socket.Disconnect(error);
}

void DNA_TCPConnectionImp::Reset(void)
{
	socket.Reset();
	destination = (DNA_IPPoint *) 0;
}

GPL_Boolean DNA_TCPConnectionImp::Open(DNA_Error & error)
{
	if (socket.Open(error))
	{
		// Set some socket options, ignoring the return
		// values since failure is not really failure.

		// Turn off blocking.
		long blocking = FALSE;
		socket.Ioctl(FIONBIO, &blocking);

		// Allow reuse of addresses.
		int reuse = TRUE;
		socket.SetOptions(SOL_SOCKET, SO_REUSEADDR, &reuse, sizeof(reuse));

		// Turn off buffering.
		IP_PROTOCOL protocol = 0;
		if (DNA_IPPoint::GetProtocol("tcp", protocol))
		{
			int noDelay = TRUE;
			socket.SetOptions(protocol, TCP_NODELAY, &noDelay, sizeof(noDelay));
		}
		return TRUE;
	}
	return FALSE;
}

GPL_Boolean DNA_TCPConnectionImp::Bind(DNA_Point *point, DNA_Error & error)
{
	return socket.Bind(point, error);
}

GPL_Boolean DNA_TCPConnectionImp::Close(DNA_Error & error)
{
	return socket.Close(error);
}

GPL_Boolean DNA_TCPConnectionImp::Read(void *buffer, const unsigned long size,
										unsigned long & length,
										DNA_Error & error)
{
	return socket.Read(buffer, size, length, error);
}

GPL_Boolean DNA_TCPConnectionImp::Read(void *buffer, const unsigned long size,
										const long timeout,
										unsigned long & length,
										DNA_Error & error)
{
	return socket.Read(buffer, size, timeout, length, error);
}

GPL_Boolean DNA_TCPConnectionImp::Write(void *buffer, const unsigned long size,
										unsigned long & length,
										DNA_Error & error)
{
	return socket.Write(buffer, size, length, error);
}

GPL_OBJECT_IMPLEMENTATION(DNA_TCPConnectionImp, 0x40070005, "DNA_TCPConnectionImp");

