
/******************************************************************************

Coral Tree dna/udp.cc
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include <gpl/gpl.h>
#include <dna/dna.h>

/******************************************************************************
class DNA_UDPBindingImp
******************************************************************************/
DNA_UDPBindingImp::DNA_UDPBindingImp(void) : destination((DNA_IPPoint *) 0)
{
}

DNA_UDPBindingImp::~DNA_UDPBindingImp(void)
{
}

long DNA_UDPBindingImp::Compare(const DNA_BusImp *imp) const
{
	if (GetClassId() != imp->GetClassId())
	{
		return GetClassId() - imp->GetClassId();
	}
	return socket.GetSocketHandle() - ((DNA_UDPBindingImp *) imp)->socket.GetSocketHandle();
}

void DNA_UDPBindingImp::SetDestination(DNA_Point *p)
{
	if ((DNA_Point *) 0 != p)
	{
		if (DNA_IPPoint::GetClassIdStatic() == p->GetClassId())
		{
			destination = (DNA_IPPoint *) p;
		}
	}
}

void DNA_UDPBindingImp::Reset(void)
{
	socket.Reset();
	destination = (DNA_IPPoint *) 0;
}

GPL_Boolean DNA_UDPBindingImp::Open(DNA_Error & error)
{
	if (socket.Open(error))
	{
		// Set some socket options, ignoring the return
		// values since failure is not really failure.

		// Turn off blocking.
		long blocking = FALSE;
		socket.Ioctl(FIONBIO, &blocking);

		// Allow reuse of addresses.
		int reuse = TRUE;
		socket.SetOptions(SOL_SOCKET, SO_REUSEADDR, &reuse, sizeof(reuse));
		return TRUE;
	}
	return FALSE;
}

GPL_Boolean DNA_UDPBindingImp::Bind(DNA_Point *point, DNA_Error & error)
{
	if ((DNA_Point *) 0 == point)
	{
		error.Set(DNA_ERROR_BAD_ADDRESS, 0);
		return FALSE;
	}

	return socket.Bind(point, error);
}

GPL_Boolean DNA_UDPBindingImp::Close(DNA_Error & error)
{
	return socket.Close(error);
}

GPL_Boolean DNA_UDPBindingImp::Read(void *buffer, const unsigned long size,
									unsigned long & length, DNA_Error & error)
{
	if ((DNA_Point *) 0 == destination)
	{
		error.Set(DNA_ERROR_BAD_ADDRESS, 0);
		return FALSE;
	}

	return socket.ReadFrom(destination, buffer, size, length, error);
}

GPL_Boolean DNA_UDPBindingImp::Read(void *buffer, const unsigned long size,
									const long timeout, unsigned long & length,
									DNA_Error & error)
{
	if ((DNA_Point *) 0 == destination)
	{
		error.Set(DNA_ERROR_BAD_ADDRESS, 0);
		return FALSE;
	}

	return socket.ReadFrom(destination, buffer, size, timeout, length, error);
}

GPL_Boolean DNA_UDPBindingImp::Write(void *buffer, const unsigned long size,
									unsigned long & length, DNA_Error & error)
{
	if ((DNA_IPPoint *) 0 == destination)
	{
		error.Set(DNA_ERROR_BAD_ADDRESS, 0);
		return FALSE;
	}

	return socket.WriteTo(destination, buffer, size, length, error);
}

GPL_OBJECT_IMPLEMENTATION(DNA_UDPBindingImp, 0x40070006, "DNA_UDPBindingImp");

