
/******************************************************************************

COPYRIGHT_HEADER

******************************************************************************/

#include <dna/dna.h>

//#define TEST_ADDR 	"227.9.18.27"
#define TEST_ADDR 	"224.9.18.27"
#define TEST_PORT	3410


class DNA_TestBus : public DNA_Bus
{
	public:
		DNA_TestBus();
virtual	~DNA_TestBus();
		long	Peek()
		{
			return rmf_imp->GetReceiver()->Peek();
		}
		void PrintIDs()
		{
			//rmf_imp.GetSender()->PrintIDs();
		}

	public:
		DNA_RMF_Group_BusImp	*rmf_imp;
		DNA_Error				lastError;
};

DNA_TestBus	*DNA_TEST_BUS;

class RAMP_Accept : public RAMP_AcceptHandler
{
	void ReceiveCallback(INT32 arg, UINT32 owner)
	{
		fprintf(stderr, "[RAMP_AcceptHandler::ReceiveCallback]\n");
		DNA_TEST_BUS->PrintIDs();
	}
};

class RAMP_Leave : public RAMP_LeaveHandler
{
	void ReceiveCallback(INT32 arg, UINT32 owner)
	{
		fprintf(stderr, "[RAMP_LeaveHandler::ReceiveCallback]\n");
		DNA_TEST_BUS->PrintIDs();
	}
};

class DNA_XBus_Shell : public WBA_Shell
{
	public:
		DNA_XBus_Shell();
virtual	~DNA_XBus_Shell();

virtual GPL_Boolean Initialize(void);
virtual void Finalize(void);
virtual	void Process(void);
	private:
		DNA_TestBus		testBus;
};

class DNA_TestWriteHandler : public WBA_CommandHandler
{
	public:
		DNA_TestWriteHandler()
		{
			names.AppendElement("write");
		}
virtual	~DNA_TestWriteHandler(){}

const char *GetUsage(void) const
		{
			return "<size>";
		}
const char *GetDescription(void) const
		{
			return "write 'size' bytes to bus";
		}
		GPL_Boolean HandleCommand(void)
		{
			if(arguments.GetCount() < 2) return FALSE;
			unsigned long size;
			size = arguments.GetElement(1).ToLong();
			void *buffer;
			buffer = (void *)osdMalloc(size);
			unsigned long length;
			DNA_TEST_BUS->Write(buffer, size, length);
			osdFree(buffer);
			wbaLog("DNA LOG", "wrote %u bytes", length);
			wbaLog("DNA LOG", "%u bytes available", DNA_TEST_BUS->Peek());
		}
};

class DNA_TestReadHandler : public WBA_CommandHandler
{
	public:
		DNA_TestReadHandler()
		{
			names.AppendElement("read");
		}
virtual	~DNA_TestReadHandler(){}

const char *GetUsage(void) const
		{
			return "<size>";
		}
const char *GetDescription(void) const
		{
			return "read 'size' bytes from bus";
		}
		GPL_Boolean HandleCommand(void)
		{
			if(arguments.GetCount() < 2) return FALSE;
			unsigned long size;
			size = arguments.GetElement(1).ToLong();
			void *buffer;
			buffer = (void *)osdMalloc(size);
			unsigned long length;
			DNA_TEST_BUS->Read(buffer, size, length);
			osdFree(buffer);
			wbaLog("DNA LOG", "read %u bytes", length);
			wbaLog("DNA LOG", "%u bytes available", DNA_TEST_BUS->Peek());
		}
};


DNA_TestBus::DNA_TestBus()
{
	DNA_TEST_BUS = this;

	rmf_imp = new DNA_RMF_Group_BusImp();
	SetImp(rmf_imp);

	IP_HOST_ADDRESS	address;
	GPL_String string;
	string = TEST_ADDR;
	if(!dnaIpHostAddressParse(string, address))
	{
		wbaLog("DNA ERROR", "could not parse address");
	}

	DNA_GroupPoint point;
	point.SetAddress(address);
	point.SetPort(TEST_PORT);
	rmf_imp->SetGroupPoint(point);

	rmf_imp->Open(lastError);
}

DNA_TestBus::~DNA_TestBus()
{
	rmf_imp->Close(lastError);
}

DNA_XBus_Shell::DNA_XBus_Shell() : WBA_Shell((WBA_Shell *)this) 
{
}

DNA_XBus_Shell::~DNA_XBus_Shell()
{
}

GPL_Boolean DNA_XBus_Shell::Initialize(void)
{
	AddCommand(new WBA_CommandHelp(this));
	AddCommand(new WBA_CommandQuit(this));
	AddCommand(new DNA_TestWriteHandler());
	AddCommand(new DNA_TestReadHandler());

	return TRUE;
}

void DNA_XBus_Shell::Finalize(void)
{
}

void DNA_XBus_Shell::Process(void)
{
}

int main(const int argc, const char **argv, const char **envp)
{
	DNA_XBus_Shell shell;

	{
		GPL_Aware gplAware;
		WBA_Aware wbaAware;
		DNA_Aware dnaAware;

		// Load only relevant environment variables.
		WBA_ENV.LoadEnvVariables(envp, FALSE);
	}

	return shell.Main();
}

