/******************************************************************************

Coral Tree dna/xdnasrvr.cc
Copyright (C) 1998 Infinite Monkeys Incorporated

This program is free software; you can redistribute it and/or modify
it under the terms of the Artistic License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
Artistic License for more details.

You should have received a copy of the Artistic License along with
this program (see meta/ART_LICE); if not, write to coral@imonk.com.

******************************************************************************/

#include <gpl/gpl.h>
#include <wba/wba.h>
#include <dna/dna.h>

enum DNA_TestOperation
{
	DNA_TEST_NOP			= 0x00,
	DNA_TEST_PING		= 0x01,
	DNA_TEST_SHUTDOWN	= 0xFF
};

#define DNA_SERVER_TO_USE DNA_ThreadedServer

/***********************************************************************
class DNA_TestServer

***********************************************************************/
class DNA_TestServer : public WBA_Daemon, public DNA_SERVER_TO_USE
{
	public:
		DNA_TestServer(void);
		virtual ~DNA_TestServer(void);

		virtual const char * GetName(void) const;
		virtual const char * GetParameterUsage(void) const;

		virtual GPL_Boolean Initialize(void);
		virtual void Finalize(void);

	protected:
		virtual void Daemonize(void);
		virtual void Process(void);
		virtual void ProcessMessage(DNA_Message & message, const GPL_Opaque key);
};

/***********************************************************************
DNA_TestServer::DNA_TestServer
***********************************************************************/
DNA_TestServer::DNA_TestServer(void) : WBA_Daemon(this)
{
}

/***********************************************************************
DNA_TestServer::~DNA_TestServer
***********************************************************************/
DNA_TestServer::~DNA_TestServer(void)
{
}

/******************************************************************************
const char * DNA_TestServer::GetName(void) const
******************************************************************************/
const char * DNA_TestServer::GetName(void) const
{
	static const char name[] = "DNA_TestServer";
	return name;
}

/******************************************************************************
const char * DNA_TestServer::GetParameterUsage(void) const
******************************************************************************/
const char * DNA_TestServer::GetParameterUsage(void) const
{
	static const char parameters[] = "";
	return parameters;
}

/******************************************************************************

******************************************************************************/
void DNA_TestServer::Daemonize(void)
{
	// Debugged daemons should not fork.
	if (!WBA_ARGS.Find("DNA_TEST_DEBUG")->IsTrue())
	{
		WBA_Daemon::Daemonize();
	}
}

/***********************************************************************
GPL_Boolean DNA_TestServer::Initialize(void)
***********************************************************************/
GPL_Boolean DNA_TestServer::Initialize(void)
{
	// Set the timeout.
	//SetTimeout(DNA_MULTIPLEXOR_BLOCK);

	// Set-up the source address.
	GPL_String host;
	DNA_IPPoint::GetHost(host);
	register DNA_IPPoint *source = new DNA_IPPoint();
	source->SetHost(host);
	source->SetPort((unsigned short)
						(0xFFFF & WBA_ARGS.Find("DNA_TEST_PORT")->GetLong()));
	SetSource(source);

	return DNA_SERVER_TO_USE::Initialize() && WBA_Daemon::Initialize();
}

/***********************************************************************
void DNA_TestServer::Finalize(void)
***********************************************************************/
void DNA_TestServer::Finalize(void)
{
	DNA_SERVER_TO_USE::Finalize();
	WBA_Daemon::Finalize();
}

/***********************************************************************
void DNA_TestServer::Process(void)
***********************************************************************/
void DNA_TestServer::Process(void)
{
fprintf(stderr,"Process\n");
	Wait();
}

/***********************************************************************
void DNA_TestServer::ProcessMessage(DNA_Message & message, const GPL_Opaque key)
***********************************************************************/
void DNA_TestServer::ProcessMessage(DNA_Message & message, const GPL_Opaque key)
{
	register const DNA_TestOperation op =
		(DNA_TestOperation) message.ReadUnsignedChar();
	switch (op)
	{
	case DNA_TEST_NOP:
		// NOP
		break;
	case DNA_TEST_PING:
fprintf(stderr,"DNA_TEST_PING\n");
		message.Clear();
		message.AppendUnsignedChar((const unsigned char) DNA_TEST_PING);
		message.AppendUnsignedChar((const unsigned char) TRUE);
		if (!Reply(message, key))
		{
			WBA_APP->Printf("Unable to reply to ping from %X\n", key);
		}
		break;
	case DNA_TEST_SHUTDOWN:
		if (WBA_ARGS.Find ("DNA_TEST_DEBUG")->IsTrue())
		{
			WBA_APP->Printf("Shutting down");
		}
		continueProcessing = FALSE;
		break;
	default:
		WBA_APP->Printf("Unknown operation %X", op);
		break;
	}
}

/******************************************************************************
int main(const int argc, const char **argv, const char **envp)
******************************************************************************/
int main(const int argc, const char **argv, const char **envp)
{
	DNA_TestServer server;

	// Set up the aliases a la getopt(3C).
	WBA_ARGS.Set("DNA_TEST_DEBUG", "0", "Print debugging information", 1, "d");
	WBA_ARGS.Set("DNA_TEST_PRINT_HELP", "0", "Print help and exit", 1, "h");
	WBA_ARGS.Set("DNA_TEST_PORT", "7103", "Server port", 1, "p");
	WBA_ARGS.Set("DNA_TEST_PRINT_VERSION", "0", "Print version and exit", 1, "v");

	// Provide scoping for aware objects.
	{
		GPL_Aware gplAware;
		WBA_Aware wbaAware;
		DNA_Aware dnaAware;

		// Load only relevant environment variables.
		WBA_ENV.LoadEnvVariables(envp, FALSE);
	}

	// Load all arguments.
	if (!WBA_ENV.LoadCommandLine(argc, argv, TRUE))
	{
		server.Printf("%s", server.GetUsage());
		server.SetExitCode(-1);
		server.Exit();
	}
	if (WBA_ARGS.Find("DNA_TEST_PRINT_HELP")->IsTrue())
	{
		server.Printf("%s", server.GetUsage());
		server.SetExitCode(0);
		server.Exit();
	}
	if (WBA_ARGS.Find("DNA_TEST_PRINT_VERSION")->IsTrue())
	{
		server.Printf("%s %s", server.GetName(),
						server.GetVersion().ToString().GetData());
		server.SetExitCode(0);
		server.Exit();
	}

	return server.Main();
}

